function _extends() { _extends = Object.assign ? Object.assign.bind() : function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

import React, { useMemo, useRef } from 'react';
import { Checkbox } from 'rlj-design';
import { useStoreContext } from "../hooks";
import { useFieldsToOptions } from "../hooks/useFieldsToOptions";
import { PREFIX_CLS } from "../../constants/dictionary";
import "../style/fieldCheckbox.less";
var CheckboxGroup = Checkbox.Group;

var getFilterData = function getFilterData(data, fieldIdList) {
  return (data === null || data === void 0 ? void 0 : data.filter(function (item) {
    var fieldId = item.fieldId;
    return !item.fieldDisabled || (fieldIdList === null || fieldIdList === void 0 ? void 0 : fieldIdList.includes(fieldId));
  }).map(function (item) {
    return item.fieldId;
  })) || [];
};

var FieldItem = function FieldItem(props) {
  var prefixCls = "".concat(PREFIX_CLS, "-fieldMultipleCheckbox");

  var _useStoreContext = useStoreContext(),
      checkedValueMap = _useStoreContext.checkedValueMap,
      setCheckedValueMap = _useStoreContext.setCheckedValueMap;

  var groupId = props.groupId,
      groupName = props.groupName,
      children = props.children,
      groupDisabled = props.groupDisabled,
      fieldId = props.fieldId;
  var options = useFieldsToOptions(children);
  var key = groupId || fieldId;
  var lastChecked = useRef(false);

  var groupOnChange = function groupOnChange(e) {
    if (e.length) {
      checkedValueMap.set(key, e);
    } else {
      checkedValueMap.delete(key);
    }

    setCheckedValueMap(new Map(checkedValueMap));
  };

  var checkedAllOnChange = function checkedAllOnChange(e) {
    var checked = e.target.checked;

    if (checked && !lastChecked.current) {
      var value = getFilterData(children, checkedValueMap.get(key));
      checkedValueMap.set(key, value);
      lastChecked.current = true;
    } else {
      checkedValueMap.delete(key);
      lastChecked.current = false;
    }

    setCheckedValueMap(new Map(checkedValueMap));
  };

  var _useMemo = useMemo(function () {
    var _checkedValueMap$get;

    var curLen = ((_checkedValueMap$get = checkedValueMap.get(key)) === null || _checkedValueMap$get === void 0 ? void 0 : _checkedValueMap$get.length) || 0;
    var allLen = (children === null || children === void 0 ? void 0 : children.length) || 0;
    var indeterminate = checkedValueMap.has(key) && curLen < allLen;
    var checkedAll = checkedValueMap.has(key) && !indeterminate;
    return {
      indeterminate: indeterminate,
      checkedAll: checkedAll
    };
  }, [checkedValueMap]),
      indeterminate = _useMemo.indeterminate,
      checkedAll = _useMemo.checkedAll;

  return /*#__PURE__*/React.createElement("div", {
    className: "".concat(prefixCls, "-container-item")
  }, /*#__PURE__*/React.createElement("div", {
    style: {
      width: 120
    }
  }, groupId && /*#__PURE__*/React.createElement(Checkbox, {
    disabled: groupDisabled,
    indeterminate: indeterminate,
    checked: checkedAll,
    onChange: checkedAllOnChange
  }, groupName)), /*#__PURE__*/React.createElement("div", null, /*#__PURE__*/React.createElement(CheckboxGroup, {
    options: options,
    onChange: groupOnChange,
    value: checkedValueMap.get(key)
  })));
};

export var FieldMultipleCheckbox = function FieldMultipleCheckbox() {
  var prefixCls = "".concat(PREFIX_CLS, "-fieldMultipleCheckbox");

  var _useStoreContext2 = useStoreContext(),
      checkedValueMap = _useStoreContext2.checkedValueMap,
      setCheckedValueMap = _useStoreContext2.setCheckedValueMap,
      fieldsData = _useStoreContext2.fieldsData,
      disableFields = _useStoreContext2.disableFields;

  var lastChecked = useRef(false);

  var checkedAllOnChange = function checkedAllOnChange(e) {
    var checked = e.target.checked;

    if (checked && !lastChecked.current) {
      fieldsData.forEach(function (item) {
        var children = item.children,
            groupId = item.groupId,
            fieldId = item.fieldId;
        var key = groupId || fieldId;
        var value = getFilterData(children, checkedValueMap.get(key));
        (value === null || value === void 0 ? void 0 : value.length) && checkedValueMap.set(key, value);
      });
      lastChecked.current = true;
    } else {
      fieldsData.forEach(function (item) {
        var _checkedValueMap$get2;

        var groupId = item.groupId,
            fieldId = item.fieldId;
        var key = groupId || fieldId;
        var retainFields = (_checkedValueMap$get2 = checkedValueMap.get(key)) === null || _checkedValueMap$get2 === void 0 ? void 0 : _checkedValueMap$get2.filter(function (obj) {
          return disableFields.includes(obj);
        });

        if (retainFields !== null && retainFields !== void 0 && retainFields.length) {
          checkedValueMap.set(key, retainFields);
        } else {
          checkedValueMap.delete(key);
        }
      });
      lastChecked.current = false;
    }

    setCheckedValueMap(new Map(checkedValueMap));
  };

  var _useMemo2 = useMemo(function () {
    var size = checkedValueMap.size;
    var checkedAll = fieldsData.every(function (item) {
      var _checkedValueMap$get3;

      var children = item.children,
          groupId = item.groupId,
          fieldId = item.fieldId;
      var key = groupId || fieldId;
      return ((_checkedValueMap$get3 = checkedValueMap.get(key)) === null || _checkedValueMap$get3 === void 0 ? void 0 : _checkedValueMap$get3.length) === (children === null || children === void 0 ? void 0 : children.length);
    });
    var indeterminate = size > 0 && !checkedAll;
    return {
      indeterminate: indeterminate,
      checkedAll: checkedAll
    };
  }, [checkedValueMap]),
      indeterminate = _useMemo2.indeterminate,
      checkedAll = _useMemo2.checkedAll;

  return /*#__PURE__*/React.createElement("div", {
    className: prefixCls
  }, /*#__PURE__*/React.createElement("div", {
    className: "".concat(prefixCls, "-header")
  }, /*#__PURE__*/React.createElement(Checkbox, {
    style: {
      width: 122
    },
    indeterminate: indeterminate,
    checked: checkedAll,
    onChange: checkedAllOnChange
  }, "\u5168\u9009\u5206\u7EC4"), /*#__PURE__*/React.createElement("span", {
    style: {
      marginTop: 3
    }
  }, "\u5B57\u6BB5")), /*#__PURE__*/React.createElement("div", {
    className: "".concat(prefixCls, "-container")
  }, fieldsData === null || fieldsData === void 0 ? void 0 : fieldsData.map(function (item, index) {
    return /*#__PURE__*/React.createElement(FieldItem, _extends({
      key: index
    }, item));
  })));
};